/*
 		Gravity/Momentum Part 3 tutorial
 		by: Steve Wortham
 		website: www.GLdomain.com
 		email: steve@gldomain.com
		
		last updated 9/1/2001
*/

/*
 NOTE:  SET THE ACTIVE CONFIGURATION FOR VISUAL C++ TO RELEASE.  
 THE PROGRAM WILL NOT RUN PROPERLY UNDER DEBUG MODE.  IF YOU KNOW HOW TO FIX THIS
 PROBLEM, PLEASE EMAIL ME AT STEVE@GLDOMAIN.COM
*/


#pragma warning(disable: 4244)  // DISABLE CONVERSION WARNINGS IN VISUAL C++

// WINDOWS & OPENGL STUFF //
#include <math.h>		                                    // Header File For Math functions
#include <windows.h>										// Header File For Windows
#include <stdlib.h>											// Header File For Standard Input / Output
#include <gl\gl.h>											// Header File For The OpenGL32 Library
#include <gl\glaux.h>										// Header File For The Glaux Library

HDC		hDC=NULL;			// Private GDI Device Context
HGLRC	hRC=NULL;			// Permanent Rendering Context
HWND	hWnd=NULL;			// Holds Our Window Handle
bool	keys[256];			// Array Used For The Keyboard Routine
bool	active=TRUE;		// Window Active Flag Set To TRUE By Default
bool	fullscreen=TRUE;	// Fullscreen Flag Set To Fullscreen Mode By Default
int     SCREEN_WIDTH = 800;
int     SCREEN_HEIGHT = 600;
LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);	// Declaration For WndProc
//////////////////////////// 

#define MAX_PARTICLES 100 // # of particles

typedef struct						// Create A Structure For Particle
{
	GLfloat	x;						// X Position
	GLfloat	y;						// Y Position
	GLfloat	xi;						// X Direction
	GLfloat	yi;						// Y Direction
	GLfloat	r;						// Red
	GLfloat	g;						// Green
	GLfloat	b;						// Blue
}
particles;							// Particles Structure

particles particle[MAX_PARTICLES];	// Particle Array (Room For Particle Info)

GLfloat particle_size = .1f;

GLuint	texture[1];
int loop, loop2;
GLfloat V, Angle;
GLfloat default_r = .75f;
GLfloat default_g = .2f;
GLfloat default_b = .1f;
GLfloat x = 0;
GLfloat y = 5;
GLfloat xp = 1; 
GLfloat yp = 0;
GLfloat dist = 0;
GLfloat slowdown= 500;

GLfloat angle, Hypoten;

GLfloat mouse_x = 0;
GLfloat mouse_y = 0;
bool mouse_click = false;
GLuint CROSSHAIR;

GLdouble Time1;
GLdouble Time2;
GLdouble DiffTime;
GLdouble FPS = 0;

// Create A Structure For The Timer Information
struct
{
  __int64       frequency;          // Timer Frequency
  GLdouble            resolution;          // Timer Resolution
  unsigned long mm_timer_start;     
  
  // Multimedia Timer Start Value
  unsigned long mm_timer_elapsed;      // Multimedia Timer Elapsed Time
  bool   performance_timer;    
  
  // Using The Performance Timer?
  __int64       performance_timer_start;      // Performance Timer Start Value
  __int64       performance_timer_elapsed; // Performance Timer Elapsed Time
} timer;

// Initialize Our Timer
inline void TimerInit(void)
{
     memset(&timer, 0, sizeof(timer));   
 // Clear Our Timer Structure
     // Check To See If A Performance Counter Is Available
     // If One Is Available The Timer Frequency Will Be Updated
     if (!QueryPerformanceFrequency((LARGE_INTEGER *) &timer.frequency))
     {
          // No Performace Counter Available
          timer.performance_timer = FALSE;                      // Set Performance Timer To FALSE
          timer.mm_timer_start = timeGetTime();                 // Use timeGetTime()
          timer.resolution  = 1.0f/1000.0f;                           // Set Our Timer Resolution To .001f
          timer.frequency   = 1000;                                     // Set Our Timer Frequency To 1000
          timer.mm_timer_elapsed = timer.mm_timer_start; // Set The Elapsed Time
     }
     else
     {
          // Performance Counter Is Available, Use It Instead Of The Multimedia Timer
          // Get The Current Time And Store It In performance_timer_start
          QueryPerformanceCounter((LARGE_INTEGER *) &timer.performance_timer_start);
          timer.performance_timer   = TRUE;    // Set Performance Timer To TRUE
          // Calculate The Timer Resolution Using The Timer Frequency
          timer.resolution    = (GLdouble) (((double)1.0f)/((double)timer.frequency));
          // Set The Elapsed Time To The Current Time
          timer.performance_timer_elapsed = timer.performance_timer_start;
     }
}

// Get Time In Milliseconds
inline GLdouble TimerGetTime()
{
     __int64 time;                                  // 'time' Will Hold A 64 Bit Integer
     if (timer.performance_timer)           // Are We Using The Performance Timer?
     {
          QueryPerformanceCounter((LARGE_INTEGER *) &time); // Current Performance Time
          // Return The Time Elapsed since TimerInit was called
          return ( (GLdouble) ( time - timer.performance_timer_start) * timer.resolution)*1000.0f;
     }
     else
     {
          // Return The Time Elapsed since TimerInit was called
          return ( (GLdouble) ( timeGetTime() - timer.mm_timer_start) * timer.resolution)*1000.0f;
     }
}

// This function will make the blurry texture perfect for the particles.  
// Sorry for the lack of comments in it(some of it confuses me too).
inline int setCheckTexture(void)
{
    int texWidth = 256;
    int texHeight = 256;
    GLubyte *texPixels, *p;
    int texSize;
    int i, j;
//    int radius;

    texSize = texWidth*texHeight*4*sizeof(GLubyte);
    texPixels = (GLubyte *) malloc(texSize);
    if (texPixels == NULL) 
    {
	  return false;
    }

    p = texPixels;
    for (i=0; i<texHeight; ++i) 
    {
	  for (j=0; j<texWidth; ++j) 
      {
	    GLuint dist = hypot(GLfloat(i - (texHeight / 2)),GLfloat(j - (texWidth / 2)));
        
        GLfloat color = 255-(dist*1.8);
        if (color < 0) color = 0;
        p[0] = color;
        p[1] = color;
        p[2] = color;
        p[3] = color;   
        p+=4;
	  }
    }

	glBindTexture(GL_TEXTURE_2D, texture[0]);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, texWidth, texHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, texPixels);

    free(texPixels);
    
    return true;
}

inline GLvoid ReSizeGLScene(GLsizei width, GLsizei height)		// Resize And Initialize The GL Window
{
	if (height==0)										// Prevent A Divide By Zero By
	{
		height=1;										// Making Height Equal One
	}

	glViewport(0,0,width,height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

	SCREEN_WIDTH = width;
	SCREEN_HEIGHT = height;
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
}


inline void make_crosshair()
{
  float crosshair_size = .25f;
  
  CROSSHAIR=glGenLists(1);									
  glNewList(CROSSHAIR,GL_COMPILE);							
  
  glDisable(GL_TEXTURE_2D);
  glColor4f(1,0,0,1);
  glBegin(GL_LINES);
  glVertex2f(-crosshair_size,0);
  glVertex2f( crosshair_size,0);
  glVertex2f(0,-crosshair_size);
  glVertex2f(0, crosshair_size);
  glEnd();
  glEnable(GL_TEXTURE_2D);

  glEndList();	
}

inline int InitGL(GLvoid)										// All Setup For OpenGL Goes Here
{
    if (!setCheckTexture())
    {
      return false;
    }	
	glLoadIdentity();									// Reset The Projection Matrix
    glEnable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    
	glPointSize(5);  // Set pixel size to 5x5

//  ***** COMMENT THIS NEXT LINE IF IT RUNS TOO SLOW *****  //  
	glEnable(GL_POINT_SMOOTH);  
//  ******************************************************  //
    
    for (loop=0;loop<MAX_PARTICLES;loop++)				// Initializes All The Textures
	{
      GLfloat D = GLfloat(rand()%1000)/100;  // Distance from center for the particle 
      Angle = GLfloat(rand()%360);  // Angle of the particle

      particle[loop].x = sin(Angle) * D;  // Set X position
      particle[loop].y = cos(Angle) * D;  // Set Y position

      particle[loop].xi = 0;  // Set X velocity
      particle[loop].yi = 0;  // Set Y velocity

      particle[loop].r = default_r;  // Set Red Value
      particle[loop].g = default_g;    // Set Green Value
      particle[loop].b = default_b;    // Set Blue Value
	}

	make_crosshair();
	
    TimerInit(); //initialize timer
	Time1 = TimerGetTime()/1000;
    Time2 = TimerGetTime()/1000;

    return true;
}

inline GLfloat ABS(GLfloat A)
{
  if (A < 0)
    return -A;
  else 
    return A;
}

inline GLfloat Hypot(GLfloat a, GLfloat b)
{  
  return sqrt(a*a+b*b);
}
   
inline GLfloat Reciprocal(GLfloat n)
{ 
  if (n < .1f && n >= 0)  // prevent devision of 0
  {
    return 1/.1f;
  }
  else if (n > -.1f && n < 0)  // prevent devision of 0
  {
    return 1/-.1f;
  }
    
  return 1/n;  
}

inline int DrawGLScene(GLvoid)									// Here's Where We Do All The Drawing
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glLoadIdentity();
  glTranslatef(mouse_x,-mouse_y,-15);  
  glCallList(CROSSHAIR);  // Draw our crosshair mouse cursor

  glLoadIdentity();
  glTranslatef(0,0,-15);

  for (loop=0; loop<MAX_PARTICLES; loop++) // Loop Through All The Particles
  {
    GLfloat x=particle[loop].x; // Grab Our Particle X Position
    GLfloat y=particle[loop].y; // Grab Our Particle Y Position

    for (loop2=0; loop2<MAX_PARTICLES; loop2++) // Loop Through All The Particles for our gravity calculations
	{
	  if (loop2 != loop)
	  {
        GLfloat x2=particle[loop2].x; // Grab Our Particle X Position
        GLfloat y2=particle[loop2].y; // Grab Our Particle Y Position
        Hypoten = Hypot(x-x2,y-y2);
        if (x < x2)
          angle = -acos(-(y-y2)/-Hypoten);
        else
          angle = acos((y-y2)/Hypoten);
  
        dist = ABS(Reciprocal(Hypoten))/50;
  
        particle[loop].r += dist/MAX_PARTICLES*20;  // Make particle brighter and yellower	depending on the distance
        particle[loop].g += dist/MAX_PARTICLES*10;  // Make particle brighter and yellower	depending on the distance
        particle[loop].b += dist/MAX_PARTICLES*2;  // Make particle brighter and yellower	depending on the distance

		particle[loop].xi -= sin(angle)*(dist);  //set particle[loop].xi according to distance from x2(another particle's x coord)
        particle[loop].yi -= cos(angle)*(dist);  //set particle[loop].yi according to distance from y2(another particle's y coord)
	  }
    }
    
	if (mouse_click)
	{
      Hypoten = Hypot(x-mouse_x,y-(-mouse_y));
      if (x < mouse_x)
        angle = -acos(-(y-(-mouse_y))/-Hypoten);
      else
        angle = acos((y-(-mouse_y))/Hypoten);
  
      dist = ABS(Reciprocal(Hypoten))*5;
  
	  particle[loop].xi*=.95f; // Slow down particles  
	  particle[loop].yi*=.95f; // Slow down particles   	  

      particle[loop].xi -= sin(angle)*(dist);  //set particle[loop].xi according to distance from x2(another particle's x coord)
      particle[loop].yi -= cos(angle)*(dist);  //set particle[loop].yi according to distance from y2(another particle's y coord)
	}

	particle[loop].r *= .9f;
	particle[loop].g *= .9f;
	particle[loop].b *= .9f;
	if (particle[loop].r < default_r) particle[loop].r = default_r;
	if (particle[loop].g < default_g) particle[loop].g = default_g;
	if (particle[loop].b < default_b) particle[loop].b = default_b;
	// Set Color according to our defined rgb values specific to each particle
	glColor4f(particle[loop].r,particle[loop].g,particle[loop].b,1);

    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(1,1);  glVertex2f(x+particle_size,y+particle_size);  
    glTexCoord2f(1,0);  glVertex2f(x+particle_size,y-particle_size);  
    glTexCoord2f(0,1);  glVertex2f(x-particle_size,y+particle_size);  
    glTexCoord2f(0,0);  glVertex2f(x-particle_size,y-particle_size);  
    glEnd();       

	// *** Necessary to keep particles close to the center of the screen *** //
	if (particle[loop].x > 8) 
	{
		particle[loop].xi -= x/slowdown*25;
		particle[loop].xi *= .95f;
	}	
	else if (particle[loop].x < -8) 
	{
		particle[loop].xi -= x/slowdown*25;
		particle[loop].xi *= .95f; 
	}
	if (particle[loop].y > 6) 
	{
		particle[loop].yi -= y/slowdown*25;
		particle[loop].yi *= .95f; 
	}
	else if (particle[loop].y < -6) 
	{
		particle[loop].yi -= y/slowdown*25;
		particle[loop].yi *= .95f; 
	}
	// ********************************************************************* //
	
	particle[loop].x+=particle[loop].xi/slowdown;  // Move On The X Axis By X Speed
    particle[loop].y+=particle[loop].yi/slowdown;  // Move On The Y Axis By Y Speed
  }

  // *** CHEAP WAY OF LIMITING THE FRAMERATE *** //
  Time2 = TimerGetTime()/1000;
  while (ABS(Time2-Time1) < .015f) // .015f == 66 frames per second, .01f == 100 frames per second
  { 
    Time2 = TimerGetTime()/1000; 
  } 	  
  Time1 = TimerGetTime()/1000;
  // ******************************************* // 
  
  return TRUE;										// Keep Going
}

inline GLvoid KillGLWindow(GLvoid)								// Properly Kill The Window
{
	if (hRC)											// Do We Have A Rendering Context?
	{
		if (!wglMakeCurrent(NULL,NULL))					// Are We Able To Release The DC And RC Contexts?
		{
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		if (!wglDeleteContext(hRC))						// Are We Able To Delete The RC?
		{
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;										// Set RC To NULL
	}

	if (hDC && !ReleaseDC(hWnd,hDC))					// Are We Able To Release The DC
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;										// Set DC To NULL
	}

	if (hWnd && !DestroyWindow(hWnd))					// Are We Able To Destroy The Window?
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;										// Set hWnd To NULL
	}

	if (fullscreen)										// Are We In Fullscreen Mode?
	{
		ChangeDisplaySettings(NULL,0);					// If So Switch Back To The Desktop
		ShowCursor(TRUE);								// Show Mouse Pointer
	}
}

/*	This Code Creates Our OpenGL Window.  Parameters Are:					*
 *	title			- Title To Appear At The Top Of The Window				*
 *	width			- Width Of The GL Window Or Fullscreen Mode				*
 *	height			- Height Of The GL Window Or Fullscreen Mode			*
 *	bits			- Number Of Bits To Use For Color (8/16/24/32)			*
 *	fullscreenflag	- Use Fullscreen Mode (TRUE) Or Windowed Mode (FALSE)	*/
 
inline BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;			// Holds The Results After Searching For A Match
	HINSTANCE	hInstance;				// Holds The Instance Of The Application
	WNDCLASS	wc;						// Windows Class Structure
	DWORD		dwExStyle;				// Window Extended Style
	DWORD		dwStyle;				// Window Style

	fullscreen=fullscreenflag;			// Set The Global Fullscreen Flag

	hInstance			= GetModuleHandle(NULL);				// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// WndProc Handles Messages
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Set The Instance
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);			// Load The Default Icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Load The Arrow Pointer
	wc.hbrBackground	= NULL;									// No Background Required For GL
	wc.lpszMenuName		= NULL;									// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";								// Set The Class Name

	if (!RegisterClass(&wc))									// Attempt To Register The Window Class
	{
		MessageBox(NULL,"Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}
	
	if (fullscreen)												// Attempt Fullscreen Mode?
	{
		DEVMODE dmScreenSettings;								// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));	// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);		// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;				// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;				// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;					// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode.
			if (MessageBox(NULL,"The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","NeHe GL",MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
			{
				fullscreen=FALSE;		// Windowed Mode Selected.  Fullscreen = FALSE
			}
			else
			{
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL,"Program Will Now Close.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;									// Return FALSE
			}
		}
	}

	if (fullscreen)												// Are We Still In Fullscreen Mode?
	{
		dwExStyle=WS_EX_APPWINDOW;								// Window Extended Style
		dwStyle=WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;	// Windows Style
		ShowCursor(FALSE);										// Hide Mouse Pointer
	}
	else
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;						// Window Extended Style
		dwStyle=WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;	// Windows Style
	}

	// Create The Window
	if (!(hWnd=CreateWindowEx(	dwExStyle,			// Extended Style For The Window
								"OpenGL",			// Class Name
								title,				// Window Title
								dwStyle,			// Window Style
								0, 0,				// Window Position
								width, height,		// Selected Width And Height
								NULL,				// No Parent Window
								NULL,				// No Menu
								hInstance,			// Instance
								NULL)))				// Dont Pass Anything To WM_CREATE
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	static	PIXELFORMATDESCRIPTOR pfd=				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support GLfloat Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		bits,										// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};
	
	if (!(hDC=GetDC(hWnd)))							// Did We Get A Device Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	// Did Windows Find A Matching Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))		// Are We Able To Set The Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))				// Are We Able To Get A Rendering Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))					// Try To Activate The Rendering Context
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	ShowWindow(hWnd,SW_SHOW);						// Show The Window
	SetForegroundWindow(hWnd);						// Slightly Higher Priority
	SetFocus(hWnd);									// Sets Keyboard Focus To The Window
	ReSizeGLScene(width, height);					// Set Up Our Perspective GL Screen

	if (!InitGL())									// Initialize Our Newly Created GL Window
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	return TRUE;									// Success
}

inline LRESULT CALLBACK WndProc(	HWND	hWnd,			// Handle For This Window
							UINT	uMsg,			// Message For This Window
							WPARAM	wParam,			// Additional Message Information
							LPARAM	lParam)			// Additional Message Information
{
	switch (uMsg)									// Check For Windows Messages
	{
		case WM_ACTIVATE:							// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))					// Check Minimization State
			{
				active=TRUE;						// Program Is Active
			}
			else
			{
				active=FALSE;						// Program Is No Longer Active
			}

			return 0;								// Return To The Message Loop
		}

		case WM_SYSCOMMAND:							// Intercept System Commands
		{
			switch (wParam)							// Check System Calls
			{
				case SC_SCREENSAVE:					// Screensaver Trying To Start?
				case SC_MONITORPOWER:				// Monitor Trying To Enter Powersave?
				return 0;							// Prevent From Happening
			}
			break;									// Exit
		}

		case WM_CLOSE:								// Did We Receive A Close Message?
		{
			PostQuitMessage(0);						// Send A Quit Message
			return 0;								// Jump Back
		}

		case WM_KEYDOWN:							// Is A Key Being Held Down?
		{
			keys[wParam] = TRUE;					// If So, Mark It As TRUE
			return 0;								// Jump Back
		}

		case WM_KEYUP:								// Has A Key Been Released?
		{
			keys[wParam] = FALSE;					// If So, Mark It As FALSE
			return 0;								// Jump Back
		}

		case WM_SIZE:								// Resize The OpenGL Window
		{
			ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));  // LoWord=Width, HiWord=Height
			return 0;								// Jump Back
		}
		case WM_MOUSEMOVE:
		{
            mouse_x = float((float)((float)LOWORD(lParam)-(SCREEN_WIDTH/2)) / 50);
            mouse_y = float((float)((float)HIWORD(lParam)-(SCREEN_HEIGHT/2)) / 50);		
		}		
	}

	if (uMsg == WM_LBUTTONDOWN)
      mouse_click	= true;
	else if (uMsg == WM_LBUTTONUP) 
      mouse_click	= false;
	
	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}


int WINAPI WinMain(	HINSTANCE	hInstance,			// Instance
					HINSTANCE	hPrevInstance,		// Previous Instance
					LPSTR		lpCmdLine,			// Command Line Parameters
					int			nCmdShow)			// Window Show State
{
	MSG		msg;									// Windows Message Structure
	BOOL	done=FALSE;								// Bool Variable To Exit Loop
    
	// Ask The User Which Screen Mode They Prefer
	if (MessageBox(NULL,"Would You Like To Run In Fullscreen Mode?", "Start FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDNO)
	{
		fullscreen=FALSE;							// Windowed Mode
	}

	// Create Our OpenGL Window
	if (!CreateGLWindow("Steve's Gravity/Momentum Tutorial #3. GLdomain.com",SCREEN_WIDTH,SCREEN_HEIGHT,16,fullscreen))
	{
		return 0;									// Quit If Window Was Not Created
	}

	while(!done)									// Loop That Runs While done=FALSE
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))	// Is There A Message Waiting?
		{
			if (msg.message==WM_QUIT)				// Have We Received A Quit Message?
			{
				done=TRUE;							// If So done=TRUE
			}
			else									// If Not, Deal With Window Messages
			{
				TranslateMessage(&msg);				// Translate The Message
				DispatchMessage(&msg);				// Dispatch The Message
			}
		}
		else										// If There Are No Messages
		{
		   // Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
			if ((active && !DrawGLScene()) || keys[VK_ESCAPE])	// Active?  Was There A Quit Received?
			{			
                  done=TRUE;							// ESC or DrawGLScene Signalled A Quit
			}
			else									// Not Time To Quit, Update Screen
			{                       
  			  SwapBuffers(hDC);					// Swap Buffers (GLfloat Buffering)				                
			}
		}
	}
    
	// Shutdown
	KillGLWindow();									// Kill The Window
	return (msg.wParam);							// Exit The Program
}
